@@ -35,13 +35,12 @@ from gnomemusic.trackerwrapper import TrackerState, TrackerWrapper
 
 class CoreGrilo(GObject.GObject):
 
-    _blacklist = [
+    _blocklist = [
         'grl-bookmarks',
         'grl-filesystem',
         'grl-itunes-podcast',
         'grl-metadata-store',
-        'grl-podcasts',
-        'grl-spotify-cover'
+        'grl-podcasts'
     ]
 
     _grl_plugin_ranks = ("grl-musicbrainz-coverart:3,"
@@ -69,14 +68,11 @@ class CoreGrilo(GObject.GObject):
         self._thumbnail_sources_timeout = None
         self._wrappers = {}
 
-        self._tracker_wrapper = TrackerWrapper()
+        self._tracker_wrapper = TrackerWrapper(application)
         self._tracker_wrapper.bind_property(
             "tracker-available", self, "tracker-available",
             GObject.BindingFlags.SYNC_CREATE)
 
-        self._tracker_wrapper.connect(
-            "notify::tracker-available", self._on_tracker_available_changed)
-
         GLib.setenv("GRL_PLUGIN_RANKS", self._grl_plugin_ranks, True)
 
         Grl.init(None)
@@ -89,18 +85,42 @@ class CoreGrilo(GObject.GObject):
         self._registry.connect('source-added', self._on_source_added)
         self._registry.connect('source-removed', self._on_source_removed)
 
-        self._registry.load_all_plugins(True)
+        self._registry.load_all_plugins(False)
+
+        tracker_available_state = self._tracker_wrapper.props.tracker_available
+        if tracker_available_state != TrackerState.AVAILABLE:
+            self._tracker_wrapper.connect(
+                "notify::tracker-available",
+                self._on_tracker_available_changed)
+        else:
+            self._on_tracker_available_changed(None, None)
+
+        for plugin in self._registry.get_plugins(False):
+            plugin_id = plugin.get_id()
+            # Activate the Tracker plugin only when TrackerWrapper
+            # is available by listening to the tracker-available
+            # property, so skip it here.
+            if plugin_id != "grl-tracker3":
+                try:
+                    self._registry.activate_plugin_by_id(plugin_id)
+                except GLib.GError:
+                    self._log.debug(
+                        "Failed to activate {} plugin.".format(plugin_id))
 
         weakref.finalize(self, Grl.deinit)
 
     def _on_tracker_available_changed(self, klass, value):
-        new_state = self._tracker_wrapper.props.tracker_available
         # FIXME:No removal support yet.
+        new_state = self._tracker_wrapper.props.tracker_available
         if new_state == TrackerState.AVAILABLE:
-            tracker_plugin = self._registry.lookup_plugin("grl-tracker")
-            if tracker_plugin:
-                self._registry.unload_plugin("grl-tracker")
-            self._registry.activate_plugin_by_id("grl-tracker")
+            config = Grl.Config.new("grl-tracker3", "grl-tracker3-source")
+            config.set_string(
+                "miner-service", self._tracker_wrapper.props.miner_fs_busname)
+            config.set_string(
+                "store-path", self._tracker_wrapper.cache_directory())
+            self._registry.add_config(config)
+
+            self._registry.activate_plugin_by_id("grl-tracker3")
 
     def _on_source_added(self, registry, source):
 
@@ -112,7 +132,7 @@ class CoreGrilo(GObject.GObject):
             return GLib.SOURCE_REMOVE
 
         if ("net:plaintext" in source.get_tags()
-                or source.props.source_id in self._blacklist):
+                or source.props.source_id in self._blocklist):
             try:
                 registry.unregister_source(source)
             except GLib.GError:
@@ -128,24 +148,15 @@ class CoreGrilo(GObject.GObject):
                 self._thumbnail_sources_timeout = GLib.timeout_add_seconds(
                     5, _trigger_art_update)
 
-        new_wrapper = None
-
-        new_state = self._tracker_wrapper.props.tracker_available
-        if (source.props.source_id == "grl-tracker-source"
-                and self._tracker_wrapper.location_filter() is not None
-                and new_state == TrackerState.AVAILABLE):
-            if source.props.source_id not in self._wrappers.keys():
-                new_wrapper = GrlTrackerWrapper(
-                    source, self._application, self._tracker_wrapper)
-                self._wrappers[source.props.source_id] = new_wrapper
-                self._log.debug("Adding wrapper {}".format(new_wrapper))
-            else:
-                grl_tracker_wrapper = self._wrappers[source.props.source_id]
-                registry.unregister_source(grl_tracker_wrapper.props.source)
-                grl_tracker_wrapper.props.source = source
+        if (source.props.source_id == "grl-tracker3-source"
+                and self._tracker_wrapper.location_filter() is not None):
+            new_wrapper = GrlTrackerWrapper(
+                source, self._application, self._tracker_wrapper)
+            self._wrappers[source.props.source_id] = new_wrapper
+            self._log.debug("Adding wrapper {}".format(new_wrapper))
         elif (source.props.source_id not in self._search_wrappers.keys()
                 and source.props.source_id not in self._wrappers.keys()
-                and source.props.source_id != "grl-tracker-source"
+                and source.props.source_id != "grl-tracker3-source"
                 and source.get_supported_media() & Grl.MediaType.AUDIO
                 and source.supported_operations() & Grl.SupportedOps.SEARCH
                 and "net:internet" not in source.props.source_tags):
@@ -160,17 +171,34 @@ class CoreGrilo(GObject.GObject):
         # FIXME: Only removes search sources atm.
         self._search_wrappers.pop(source.props.source_id, None)
 
-    def get_artist_albums(self, artist, filter_model):
-        for wrapper in self._wrappers.values():
-            wrapper.get_artist_albums(artist, filter_model)
+    def get_artist_albums(self, media, filter_model):
+        """Get all album by an artist
+
+        :param Grl.Media media: A Grilo Media item that represents Artist
+        :param Gfm.FilterListModel filter_model: The model to fill
+        """
+        source = media.get_source()
+        self._wrappers[source].get_artist_albums(media, filter_model)
 
     def get_album_discs(self, media, disc_model):
-        for wrapper in self._wrappers.values():
-            wrapper.get_album_discs(media, disc_model)
+        """Get all discs from an album
+
+        :param Grl.Media media: A Grilo Media item that represents Album
+        :param Gfm.SortListModel disc_model: The model to fill
+        """
+        source = media.get_source()
+        self._wrappers[source].get_album_discs(media, disc_model)
 
     def populate_album_disc_songs(self, media, discnr, callback):
-        for wrapper in self._wrappers.values():
-            wrapper.populate_album_disc_songs(media, discnr, callback)
+        """Get all songs from an album disc
+
+        :param Grl.Media media: A Grilo Media item that represents Album
+        :param int discnr: The disc number
+        :param callback: The callback to call for every song added
+        """
+        source = media.get_source()
+        self._wrappers[source].populate_album_disc_songs(
+            media, discnr, callback)
 
     def writeback(self, media, key):
         """Store the values associated with the key.
@@ -192,29 +220,55 @@ class CoreGrilo(GObject.GObject):
                     None)
                 break
 
+    def writeback_tracker(self, media, tag):
+        """Use Tracker queries to update tags.
+
+        The tags are associated with a Tracker resource
+        (song, album, artist or external resource), so they can cannot
+        be updated with grilo writeback support.
+
+        :param Grl.Media media: A Grilo media item
+        :param str tag: tag to update
+        """
+        self._tracker_wrapper.update_tag(media, tag)
+
     def search(self, text):
         for wrapper in self._wrappers.values():
             wrapper.search(text)
         for wrapper in self._search_wrappers.values():
             wrapper.search(text)
 
-    def get_album_art_for_item(self, coresong, callback):
-        # Tracker not (yet) loaded.
-        if "grl-tracker-source" in self._wrappers:
-            self._wrappers["grl-tracker-source"].get_album_art_for_item(
-                coresong, callback)
+    def get_song_art(self, coresong):
+        """Retrieve song art for the given CoreSong
+
+        :param CoreSong coresong: CoreSong to retrieve art for
+        """
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].get_song_art(coresong)
+
+    def get_album_art(self, corealbum):
+        """Retrieve album art for the given CoreAlbum
+
+        :param CoreAlbum corealbum: CoreAlbum to retrieve art for
+        """
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].get_album_art(corealbum)
 
     def get_artist_art(self, coreartist):
-        if "grl-tracker-source" in self._wrappers:
-            self._wrappers["grl-tracker-source"].get_artist_art(coreartist)
+        """Retrieve artist art for the given CoreArtist
+
+        :param CoreArtist coreartist: CoreArtist to retrieve art for
+        """
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].get_artist_art(coreartist)
 
     def stage_playlist_deletion(self, playlist):
         """Prepares playlist deletion.
 
         :param Playlist playlist: playlist
         """
-        if "grl-tracker-source" in self._wrappers:
-            self._wrappers["grl-tracker-source"].stage_playlist_deletion(
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].stage_playlist_deletion(
                 playlist)
 
     def finish_playlist_deletion(self, playlist, deleted):
@@ -223,8 +277,8 @@ class CoreGrilo(GObject.GObject):
         :param Playlist playlist: playlist
         :param bool deleted: indicates if the playlist has been deleted
         """
-        if "grl-tracker-source" in self._wrappers:
-            self._wrappers["grl-tracker-source"].finish_playlist_deletion(
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].finish_playlist_deletion(
                 playlist, deleted)
 
     def create_playlist(self, playlist_title, callback):
@@ -233,6 +287,6 @@ class CoreGrilo(GObject.GObject):
         :param str playlist_title: playlist title
         :param callback: function to perform once, the playlist is created
         """
-        if "grl-tracker-source" in self._wrappers:
-            self._wrappers["grl-tracker-source"].create_playlist(
+        if "grl-tracker3-source" in self._wrappers:
+            self._wrappers["grl-tracker3-source"].create_playlist(
                 playlist_title, callback)
